Mod = {
    getIsMultiplayer = function() return g_currentMission.missionDynamicInfo.isMultiplayer end,
    getIsServer = function() return g_currentMission.getIsServer() end,
    getIsMasterUser = function() return g_currentMission.isMasterUser end,
    getIsValidFarmManager = function(self) return self:getIsServer() and g_currentMission.player ~= nil and g_currentMission.player.farmId ~= FarmManager.SPECTATOR_FARM_ID end,
}

Mod.dir = g_currentModDirectory
Mod.name = g_currentModName
Mod.mod = g_modManager:getModByName(Mod.name)
Mod.env = getfenv()
Mod.globalEnv = getfenv(0)

local modDescXML = loadXMLFile("modDesc", Mod.dir .. "modDesc.xml")
Mod.title = getXMLString(modDescXML, "modDesc.title.en")
Mod.author = getXMLString(modDescXML, "modDesc.author")
Mod.version = getXMLString(modDescXML, "modDesc.version")
delete(modDescXML)


--- Initialize Mod.
function Mod:init()
    local newMod = self:new()

    addModEventListener(newMod)

    return newMod
end


--- New Mod instance.
function Mod:new()
    local newMod = {}
    setmetatable(newMod, self)
    self.__index = self

    newMod.dir = g_currentModDirectory

    local modDescXML = loadXMLFile("modDesc", newMod.dir .. "modDesc.xml")
    newMod.title = getXMLString(modDescXML, "modDesc.title.en")
    newMod.author = getXMLString(modDescXML, "modDesc.author")
    newMod.version = getXMLString(modDescXML, "modDesc.version")
    delete(modDescXML)


    FSBaseMission.onStartMission = Utils.appendedFunction(FSBaseMission.onStartMission, function(baseMission, ...)
        if newMod.startMission ~= nil and type(newMod.startMission) == "function" then
            newMod:startMission(baseMission, ...)
        end
    end)

    FSBaseMission.onStartMission = Utils.prependedFunction(FSBaseMission.onStartMission, function(baseMission, ...)
        if newMod.beforeStartMission ~= nil and type(newMod.beforeStartMission) == "function" then
            newMod:beforeStartMission(baseMission, ...)
        end
    end)

    FSBaseMission.initialize = Utils.appendedFunction(FSBaseMission.initialize, function(baseMission, ...)
        if newMod.initMission ~= nil and type(newMod.initMission) == "function" then
            newMod:initMission(baseMission, ...)
        end
    end)

    FSBaseMission.loadMap = Utils.prependedFunction(FSBaseMission.loadMap, function(baseMission, ...)
        if newMod.beforeLoadMap ~= nil and type(newMod.beforeLoadMap) == "function" then
            newMod:beforeLoadMap(baseMission, ...)
        end
    end)

    FSBaseMission.loadMap = Utils.appendedFunction(FSBaseMission.loadMap, function(baseMission, ...)
        if newMod.afterLoadMap ~= nil and type(newMod.afterLoadMap) == "function" then
            newMod:afterLoadMap(baseMission, ...)
        end
    end)

    FSBaseMission.loadMapFinished = Utils.prependedFunction(FSBaseMission.loadMapFinished, function(baseMission, ...)
        if newMod.loadMapFinished ~= nil and type(newMod.loadMapFinished) == "function" then
            newMod:loadMapFinished(baseMission, ...)
        end
    end)

    FSBaseMission.loadMapFinished = Utils.appendedFunction(FSBaseMission.loadMapFinished, function(baseMission, ...)
        if newMod.afterLoadMapFinished ~= nil and type(newMod.afterLoadMapFinished) == "function" then
            newMod:afterLoadMapFinished(baseMission, ...)
        end
    end)

    return newMod
end


---Adds a new fillType
-- @param string name
-- @param string title
-- @param boolean showOnPriceTable
-- @param float pricePerLiter
-- @param float massPerliter
-- @param string hudOverlayFilename
-- @param string hudOverlayFilenameSmall
-- @param string palletFilename
-- @param table<int:float> economicCurve
-- @param string fillTypeCategory
-- @return table
function Mod:addFillType(name, title, showOnPriceTable, pricePerLiter, massPerliter, hudOverlayFilename, hudOverlayFilenameSmall, palletFilename, economicCurve)

end